/**************************************************************************/
/* File: can_sysless.h - provides can_sysdep.h definitions to hide        */
/*                      between Linux kernel and embedded target build    */
/*                                                                        */
/* LinCAN - (Not only) Linux CAN bus driver                               */
/* Copyright (C) 2002-2009 DCE FEE CTU Prague <http://dce.felk.cvut.cz>   */
/* Copyright (C) 2002-2009 Pavel Pisa <pisa@cmp.felk.cvut.cz>             */
/* Funded by OCERA and FRESCOR IST projects                               */
/*                                                                        */
/* LinCAN is free software; you can redistribute it and/or modify it      */
/* under terms of the GNU General Public License as published by the      */
/* Free Software Foundation; either version 2, or (at your option) any    */
/* later version.  LinCAN is distributed in the hope that it will be      */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty    */
/* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU    */
/* General Public License for more details. You should have received a    */
/* copy of the GNU General Public License along with LinCAN; see file     */
/* COPYING. If not, write to the Free Software Foundation, 675 Mass Ave,  */
/* Cambridge, MA 02139, USA.                                              */
/*                                                                        */
/* To allow use of LinCAN in the compact embedded systems firmware        */
/* and RT-executives (RTEMS for example), main authors agree with next    */
/* special exception:                                                     */
/*                                                                        */
/* Including LinCAN header files in a file, instantiating LinCAN generics */
/* or templates, or linking other files with LinCAN objects to produce    */
/* an application image/executable, does not by itself cause the          */
/* resulting application image/executable to be covered by                */
/* the GNU General Public License.                                        */
/* This exception does not however invalidate any other reasons           */
/* why the executable file might be covered by the GNU Public License.    */
/* Publication of enhanced or derived LinCAN files is required although.  */
/**************************************************************************/

#ifndef _CAN_SYSDEP_H
#define _CAN_SYSDEP_H


#include <cpu_def.h>
#include <malloc.h>

// typedef unsigned long atomic_t;
typedef struct { volatile int counter; } atomic_t;

#define mb()	__memory_barrier()

/**
 * container_of - cast a member of a structure out to the containing structure
 * @ptr:	the pointer to the member.
 * @type:	the type of the container struct this is embedded in.
 * @member:	the name of the member within the struct.
 *
 */
#define container_of(ptr, type, member) ({			\
	const typeof( ((type *)0)->member ) *__mptr = (ptr);	\
	(type *)( (char *)__mptr - offsetof(type,member) );})



#define wait_queue_head_t struct wait_queue *
#define wait_queue_t      struct wait_queue
#define init_waitqueue_head(queue_head) (*queue_head=NULL)
#define init_waitqueue_entry(qentry,qtask) \
		(qentry->next=NULL,qentry->task=qtask)
#define DECLARE_WAIT_QUEUE_HEAD(name) \
	struct wait_queue * name=NULL
#define DECLARE_WAITQUEUE(wait, current) \
        struct wait_queue wait = { current, NULL }

typedef irqreturn_t     can_irqreturn_t;
#define CAN_IRQ_NONE    IRQ_NONE
#define CAN_IRQ_HANDLED IRQ_HANDLED
#define CAN_IRQ_RETVAL  IRQ_RETVAL

#define CAN_IRQ_HANDLER_ARGS(irq_number, dev_id) \
		int irq_number, void *dev_id
#define can_synchronize_irq(irqnum) do {} while(0)

typedef unsigned long can_ioptr_t;
#define can_ioptr2ulong(ioaddr) ((unsigned long)(ioaddr))
#define can_ulong2ioptr(addr)   ((unsigned long)(addr))
#define can_inb(ioaddr) inb(ioaddr)
#define can_outb(data,ioaddr) outb(data,ioaddr)
#define can_inw(ioaddr) inb(ioaddr)
#define can_outw(data,ioaddr) outb(data,ioaddr)
#define can_inl(ioaddr) inb(ioaddr)
#define can_outl(data,ioaddr) outb(data,ioaddr)

#define can_readb  readb
#define can_writeb writeb
#define can_readw  readw
#define can_writew writew
#define can_readl  readl
#define can_writel writel

#define can_ioport2ioptr can_ulong2ioptr

#ifdef __HAVE_ARCH_CMPXCHG
  #define CAN_HAVE_ARCH_CMPXCHG
#endif

#define can_spinlock_t             long
#define can_spin_irqflags_t        unsigned long
/* Only one CPU is supposed on sys-less embedded target => no need for spinloks */
#define can_spin_lock(lock)        do { } while (0)
#define can_spin_unlock(lock)      do { } while (0)
/* Only one CPU is supposed on sys-less embedded target => no need for spinloks */
#define can_spin_lock_irqsave(lock,flags)      save_and_cli(flags)
#define can_spin_unlock_irqrestore(lock,flags) restore_flags(flags)
#define can_spin_lock_init         can_splck_init

#define CAN_DEFINE_SPINLOCK(x)     can_spinlock_t x = 0

static inline
void can_splck_init(can_spinlock_t *x)
{
	*x=0;
}

#define can_preempt_disable()      do { } while (0)
#define can_preempt_enable()       do { } while (0)

#if 1
#define can_enable_irq(var)        do { } while (0)
#define can_disable_irq(var)       do { } while (0)
#else
#define can_enable_irq             enable_irq
#define can_disable_irq            disable_irq
#endif

#define can_printk                 printf
#define KERN_CRIT
#define KERN_ERR

/// LINUX src: include/asm-arm/bitops.h

#define set_bit ____atomic_set_bit
#define clear_bit ____atomic_clear_bit
#define change_bit ____atomic_change_bit
#define test_and_set_bit ____atomic_test_and_set_bit
#define test_and_clear_bit ____atomic_test_and_clear_bit
#define test_and_change_bit ____atomic_test_and_change_bit
#define raw_local_irq_save(flags) save_and_cli(flags);
#define raw_local_irq_restore(flags) restore_flags(flags);

/*
 * These functions are the basis of our bit ops.
 *
 * First, the atomic bitops. These use native endian.
 */
static inline void ____atomic_set_bit(unsigned int bit, volatile unsigned long *p)
{
	unsigned long flags;
	unsigned long mask = 1UL << (bit & 31);

	p += bit >> 5;

	raw_local_irq_save(flags);
	*p |= mask;
	raw_local_irq_restore(flags);
}

static inline void ____atomic_clear_bit(unsigned int bit, volatile unsigned long *p)
{
	unsigned long flags;
	unsigned long mask = 1UL << (bit & 31);

	p += bit >> 5;

	raw_local_irq_save(flags);
	*p &= ~mask;
	raw_local_irq_restore(flags);
}

static inline void ____atomic_change_bit(unsigned int bit, volatile unsigned long *p)
{
	unsigned long flags;
	unsigned long mask = 1UL << (bit & 31);

	p += bit >> 5;

	raw_local_irq_save(flags);
	*p ^= mask;
	raw_local_irq_restore(flags);
}

static inline int
____atomic_test_and_set_bit(unsigned int bit, volatile unsigned long *p)
{
	unsigned long flags;
	unsigned int res;
	unsigned long mask = 1UL << (bit & 31);

	p += bit >> 5;

	raw_local_irq_save(flags);
	res = *p;
	*p = res | mask;
	raw_local_irq_restore(flags);

	return res & mask;
}

static inline int
____atomic_test_and_clear_bit(unsigned int bit, volatile unsigned long *p)
{
	unsigned long flags;
	unsigned int res;
	unsigned long mask = 1UL << (bit & 31);

	p += bit >> 5;

	raw_local_irq_save(flags);
	res = *p;
	*p = res & ~mask;
	raw_local_irq_restore(flags);

	return res & mask;
}

static inline int
____atomic_test_and_change_bit(unsigned int bit, volatile unsigned long *p)
{
	unsigned long flags;
	unsigned int res;
	unsigned long mask = 1UL << (bit & 31);

	p += bit >> 5;

	raw_local_irq_save(flags);
	res = *p;
	*p = res ^ mask;
	raw_local_irq_restore(flags);

	return res & mask;
}

/// LINUX src: include/asm-arm/atomic.h

#define atomic_read(v)	((v)->counter)

#define atomic_set(v,i)	(((v)->counter) = (i))

static inline int atomic_add_return(int i, atomic_t *v)
{
	unsigned long flags;
	int val;

	raw_local_irq_save(flags);
	val = v->counter;
	v->counter = val += i;
	raw_local_irq_restore(flags);

	return val;
}

static inline int atomic_sub_return(int i, atomic_t *v)
{
	unsigned long flags;
	int val;

	raw_local_irq_save(flags);
	val = v->counter;
	v->counter = val -= i;
	raw_local_irq_restore(flags);

	return val;
}

static inline int atomic_cmpxchg(atomic_t *v, int old, int new)
{
	int ret;
	unsigned long flags;

	raw_local_irq_save(flags);
	ret = v->counter;
	if (ret == old)
		v->counter = new;
	raw_local_irq_restore(flags);

	return ret;
}

#define atomic_xchg(v, new) (xchg(&((v)->counter), new))

static inline int atomic_add_unless(atomic_t *v, int a, int u)
{
	int c, old;

	c = atomic_read(v);
	while (c != u && (old = atomic_cmpxchg((v), c, c + a)) != c)
		c = old;
	return c != u;
}
#define atomic_inc_not_zero(v) atomic_add_unless((v), 1, 0)

#define atomic_add(i, v)	(void) atomic_add_return(i, v)
#define atomic_inc(v)		(void) atomic_add_return(1, v)
#define atomic_sub(i, v)	(void) atomic_sub_return(i, v)
#define atomic_dec(v)		(void) atomic_sub_return(1, v)

#define atomic_inc_and_test(v)	(atomic_add_return(1, v) == 0)
#define atomic_dec_and_test(v)	(atomic_sub_return(1, v) == 0)
#define atomic_inc_return(v)    (atomic_add_return(1, v))
#define atomic_dec_return(v)    (atomic_sub_return(1, v))
#define atomic_sub_and_test(i, v) (atomic_sub_return(i, v) == 0)

#define atomic_add_negative(i,v) (atomic_add_return(i, v) < 0)

/* Atomic operations are already serializing on ARM */
#define smp_mb__before_atomic_dec()	barrier()
#define smp_mb__after_atomic_dec()	barrier()
#define smp_mb__before_atomic_inc()	barrier()
#define smp_mb__after_atomic_inc()	barrier()

/// LINUX src: include/linux/interrupt.h

#define ATOMIC_INIT(i)	{ (i) }

/* Tasklets --- multithreaded analogue of BHs.

   Main feature differing them of generic softirqs: tasklet
   is running only on one CPU simultaneously.

   Main feature differing them of BHs: different tasklets
   may be run simultaneously on different CPUs.

   Properties:
   * If tasklet_schedule() is called, then tasklet is guaranteed
     to be executed on some cpu at least once after this.
   * If the tasklet is already scheduled, but its excecution is still not
     started, it will be executed only once.
   * If this tasklet is already running on another CPU (or schedule is called
     from tasklet itself), it is rescheduled for later.
   * Tasklet is strictly serialized wrt itself, but not
     wrt another tasklets. If client needs some intertask synchronization,
     he makes it with spinlocks.
 */

struct tasklet_struct
{
	struct tasklet_struct *next;
	unsigned long state;
	atomic_t count;
	void (*func)(unsigned long);
	unsigned long data;
};

#define DECLARE_TASKLET(name, func, data) \
struct tasklet_struct name = { NULL, 0, ATOMIC_INIT(0), func, data }

#define DECLARE_TASKLET_DISABLED(name, func, data) \
struct tasklet_struct name = { NULL, 0, ATOMIC_INIT(1), func, data }

/* CAN message timestamp source, it is called from interrupt context */
//#define can_gettimeofday do_gettimeofday

/// from linux/timer.h

struct tvec_t_base_s;

struct timer_list {
	struct list_head entry;
	unsigned long expires;

	void (*function)(unsigned long);
	unsigned long data;

	struct tvec_t_base_s *base;
#ifdef CONFIG_TIMER_STATS
	void *start_site;
	char start_comm[16];
	int start_pid;
#endif
};

void udelay(long time);

#endif /*_CAN_SYSDEP_H*/
